
/*---------------------------------------------------------------------------

                    SASEFRED Macro Library

        Name: fred_fileputds.sas  
 Description: This macro uses the data step to write the OUT=ds
              that will contain the FRED data read by already running
              the fred_filegetxml and fred_filemapxml macros. 
              The SASEFRED Interface Engine                   
       Title: Reading FactSet Data in DATA STEP from the XMLV2 datasets
              created in the SASEFRED engine assign.
     Product: SAS/ETS Software                                    
        Keys: FRED data extraction, XML FRED Data                             
       Procs: SASEFRED
       Notes: This is an uncompiled macro, used by SASEFRED to write      
              a SAS Dataset containing FRED time series data obtained
              via web URL access of the XML data, and read by the 
              XMLV2 engine into intermediate SAS datasets.
              30nov2012, saskff pulled for 13.1 copy at v940m1,
                         using s0923282.
              22may2014, saskff pulled for 14.1 copy at v940m3,
                         using s1074495.           
              20Jul2015, saskff pulled to update 15.1 to https: instead of http:
                         using S1190360. Also added connect= option per Eric Thies suggestion.

     Support: saskff
  Parameters: 

    out     - output dataset name (memname)- it is defined in the 
              folder where the physical path for the sasefred 
              libname assignment resides. 
    fredref - this is really the path you want to use on a hidden
              libref that corresponds to the engine's physical path.
     dbug   - flag to get more debug info in the SAS log about the 
              macro variables and their values. Uses PROC Printto 
              to redirect the log information just like makefile.sas 
              macro in dvtest/testauto does.
    offset  - integer, non-negative, default: 0.
    memname - string, member name for url=option.
    output  - integer, default: 1. Options: '1','2','3','4'.
    units   - string, key that indicates a data value transformation, 
              default: 'lin'.
    ids     - string, BY groups or ids that the time series are organized.
    vintage - string, used to download data as it existed on these 
              specified dates in history. They are used in conjunction 
              with output=3 or output=4.
    sort    - string, Sort results is ascending or descending order, 
              default: 'asc'.
 initNumIds - integer, initial number of time series in idlist= option.
----------------------------------------------------------------------------*/
%macro fred_fileputds( out=,fredref=,dbug=,offset=,memname=,output=,units=,ids=,vintage=,sort=,initNumIds=);

    /*-----------------------------------------------------------------------*/
    /*-- saskff: obsnvars macro to get correct nvars and nobs for merged_data --*/
    /*-----------------------------------------------------------------------*/   

       %macro obsnvars(ds);
       /*   %global dset nvars nobs;  */
          %let dset=&ds;
          %let dsid = %sysfunc(open(&dset));
          %if &dsid %then
             %do;
                %let nobs =%sysfunc(attrn(&dsid,NOBS));
                %let nvars=%sysfunc(attrn(&dsid,NVARS));
                %let rc = %sysfunc(close(&dsid));
                %put &dset has &nvars  variable(s) and &nobs observation(s).;
             %end;
          %else
             %put Open for data set &dset failed - %sysfunc(sysmsg());
       %mend obsnvars;

    /*--- main macro starts here -------------------*/
    /*--- preset status to bad for early returns ---*/

    %let totalNumIds = 1;
    %let totalVintages = 1;
    %let openCount = 0;
    %let fred_uniqueflag = 0;
    %let testout=%upcase(&out);
    %let testlen = %length(&testout);
    %if &testlen<=0 %then %goto test_skip1;     
    %let testout2 = %substr( &testout, 2 , %eval(&testlen-2) );

 %test_skip1:
    /* check for URL= option, which has predetermined name out='XFREDTPU' */
    %if &testout2=XFREDTPU %then %do;
       %let fred_uniqueflag = 1;
       %goto skip_count;
    %end;

    /* count total number of ids in the keepid[0] (ids) */
    %let len = %length(&ids);
    %if &len<=0 %then %goto vins_count;     
    %let nrbids = %substr( &ids, 2 , %eval(&len-2) );
    data _null_;
       i=0 ; 
       do until ( x = '');
       i=i+1;       
       x=scan("&nrbids",i,"");
     /*put x;                   */
     /*put i;                   */
       end ;
       call symput("totalNumIds",strip(i-1));
    run;

    /* count total number of vintage series in the vintemp string */ 
%vins_count:
    %let vinlent = %eval( %length(&vintage)-2 );   
    %if &vinlent < 0 %then %goto skip_count;
    %if &vinlent = 0 %then %do;       
        %let vintemp =; 
        %goto skip_count;
    %end;  
    %let vintemp = %substr( &vintage, 2 , &vinlent );

    data _null_;
       i=0 ; 
       do until ( x = '');
       i=i+1;       
       x=scan("&vintemp",i,",");
    /* put x;                   */
    /* put i;                   */
       end ;
       call symput("totalVintages",strip(i-1));
     run;
   
%skip_count:
    %let fred_macro_status=1;
    %let fred_macro_nobs=0;
    %let nobs=0;
    %let fred_macro_nvars=0;
    %let nvars=0;

    %let debug=&dbug;

    %if &debug=ON %then %do;
       options mprint symbolgen mlogic;
    %end;

    options validvarname=any;

    %let nrbout     = %str(&out);
    %let nrbmname   = %str(&out);  /*- may or may not have quotes -*/
    %let nrbsort    = %str(&sort);
    %let ext        = %str(sas7bdat);
    %let nrbfredref = %str(&fredref);

    %if &fred_uniqueflag = 0 %then %let myref = %str(FREDLIB);
    %else %goto fred_unique_true;  
 
    /*--------------------------------------------------------*/
    /*--- Validation of parameters supplied to macro       ---*/
    /*--------------------------------------------------------*/  
    %if &nrbout EQ %then %do;
        %put ERROR: OUT= parameter is required for FRED_FILEPUTDS macro.;
        %goto clean_up;
    %end;

    %if &nrbfredref EQ %then %do;
        %put ERROR: FREDREF= parameter is required for FRED_FILEPUTDS macro.;
        %goto clean_up;
    %end;
        
    libname FREDLIB &nrbfredref;
    /*--- remove the leading and trailing quotes from  (&nrbout) ---*/
    %let len = %length(&nrbout);
    %if &len<=0 %then %goto put_skip1;     
    %let nrbmname = %substr( &nrbout, 2 , %eval(&len-2) );

%put_skip1:
    /*--- remove the leading and trailing quotes from  (&nrbsort) ---*/
    %let len_s = %length(&nrbsort);
    %if &len_s<=0 %then %goto put_skip2;     
    %let obssort = %substr( &nrbsort, 2 , %eval(&len_s-2) );

%put_skip2:
    
    /*---------------------------------------------------------- */
    /*---NOBS and NVARS - Determine from the generated DSs ----- */    
    /*   %let dsid=%sysfunc(open(outxml.observations));          */ 
    /*   %let nobs=%sysfunc(attrn(&dsid,NOBS));                  */ 
    /*---------------------------------------------------------- */
    
    /*
    proc sql noprint;                                         
        select r.observations_count into :nobs 
        from outxml&totalNumIds..observations AS r;                         
    quit;
    %let nobs=%eval(&nobs-&offset);
    %if &nobs < 0 %then %do;
        %let nobs=0;
    %end;
    %put &nobs;                                               
    
    %if &nobs EQ 0 %then %do;
        %put ERROR: No Observations in ROWS= data set.;
        %goto clean_up;
    %end;
    */
     
    %if &totalNumIds=0 %then %do;
       %put ERROR: No Series in KEEP list, so setting the output to report only on the contents of the data.;
       %put WARNING: Output Data Set shows only the available time series, their native frequencies and range.;
       %put WARNING: Consider changing the requested frequency to allow for aggregation of observations.; 
       %goto NullKeepList;
    %end;

    %if &debug=OFF %then %do;
       options nonotes;        /*  s1111072, unresolved macros...*/
                               /*  %filename(newlog,newlog,tmp); */
       filename newlog "newlog.tmp";
       proc printto log=newlog new;
       run;
       options notes;
    %end;

    /* find the max number of observations from all databases */
    /* consult http://www2.sas.com/proceedings/sugi26/p095-26.pdf for details */
    %let nobs=0;
    %let tempnobs=0;
    %do k=1 %to &initNumIds;   /* used to be totalNumIds */
       %let dsid=%sysfunc(open(outxml&k..observation,IS));
       %if &dsid = 0 %then %do;
          %put %sysfunc(sysmsg());
          %let markNum&k = 0; /* Mark this kth dataset as unavailable */
          %put ERROR: No Observations in ROWS= data set for data set &k .;

          %if &k = &initNumIds %then %do;   /* used to be totalNumids */
              %if &openCount = 0 %then %goto clean_up;
          %end;
          %else %let nobs = 0;  /* leave nobs alone if last one isn't open */
          %goto loop_bottom;
       %end;
       %else %do;
          %let openCount = %eval(&openCount. + 1);
          %let markNum&k = &openCount; /* Mark this kth dataset as available   */
                                       /* and store its merging order in markNum*/

          %let anobs = %sysfunc(attrn(&dsid, ANOBS));
          %let whstmt = %sysfunc(attrn(&dsid, WHSTMT));        
       %end;

       %if &anobs = 1 & &whstmt = 0 %then
          %let tempnobs = %sysfunc(attrn(&dsid, NLOBS));
       %else %do;
          %if %sysfunc(getoption(msglevel)) = I %then
          %put INFO: Observations in  must be counted by iteration.;
          %let tempnobs = 0;
          %do %while (%sysfunc(fetch(&dsid)) = 0);
             %let tempnobs = %eval(&tempnobs. + 1);
          %end;
       %end;
       %let rc = %sysfunc(close(&dsid));

       %if &tempnobs > &nobs %then %let nobs=&tempnobs;
       %put tempnobs = &tempnobs;
       %put nobs = &nobs;
%loop_bottom:
    %end;

    %if &nobs EQ 0 %then %do;
        %put ERROR: No Observations in ROWS= data set.;
        %goto clean_up;
    %end;

    %let nvars = %eval(&openCount. + 1); /* used to be &totalNumIds+1 */
    /* %let nvars =1; */
    /* %put &nvars; */   

    %if &nvars EQ 0 %then %do;
        %put ERROR: No Variables in COLUMNS= data set.;
        %goto clean_up;
    %end;    

    %if &nvars<=0 %then %do;
        %put ERROR: No Variables found in OUT= data set.;  
        %goto clean_up;
        %goto screen_return;
    %end;

    /*-----------------------------------------------------------------------*/
    /*-- fasayy: quotelst macro  to put " "N around the elements of a list --*/
    /*-----------------------------------------------------------------------*/   
    %macro quotelst(strng,quote1=%str(%"),quote2=%str(%"N),delim=%str( ));
        %local i quotelst;
        %let i=1;
        %do %while(%length(%qscan(&strng,&i,|)) GT 0);
            %if %length(&quotelst) EQ 0 %then %let quotelst=&quote1.%qscan(&strng,&i,|)&quote2;
            %else %let quotelst=&quotelst.&quote1.%qscan(&strng,&i,|)&quote2;
            %let i=%eval(&i + 1);
        %if %length(%qscan(&strng,&i,|)) GT 0 %then %let quotelst=&quotelst.&delim;
        %end;
        %unquote(&quotelst)
    %mend quotelst;

    %let lenu = %length(&units);
    %let unitname = %substr( &units, 2 , %eval(&lenu-2) );


%NullKeepList:
    /* there is no time series in keep list */
    /* SHOULD CHANGE THIS LATER */
    %let  nseries_org=&initNumIds; 
    
    %if &totalNumIds=0 %then %do;       
       %if &debug=OFF %then %do;
            options nonotes;        /*  s1111072, unresolved macros...*/
                                    /*  %filename(newlog,newlog,tmp); */
            filename newlog "newlog.tmp";
            proc printto log=newlog new;
            run;
            options notes;
       %end;
  
       %if &nseries_org=1 %then %do;  /* single time serie */  
          data FREDLIB.FREDtp1(rename=(series_id=Series series_title=Title series_observation_start=Obs_Start 
                               series_observation_end=Obs_End series_frequency=Frequency 
                               series_frequency_short=Frequency_short series_units_short=units
                               series_seasonal_adjustment_short=seasonal_adjustment
                               series_popularity=popularity)
             keep=  series_id series_title series_observation_start series_observation_end series_frequency series_frequency_short series_units_short series_seasonal_adjustment_short series_popularity);
             retain series_id series_title series_observation_start series_observation_end series_frequency series_frequency_short series_units_short series_seasonal_adjustment_short series_popularity;
             set outtp1.series;
          run;

          %if &debug=ON %then %do;
              proc contents data=FREDLIB.FREDtp1; run;
              proc print data=FREDLIB.FREDtp1; run;
          %end;

          data FREDLIB.&nrbmname ;
             set FREDLIB.FREDtp1; 
          run;
          %let nobs = &nseries_org;
          %let nvars = 9;
       %end;

       %if &nseries_org>1 %then %do;  /* multiple time series */
          %do i=1 %to &nseries_org; 
             data FREDLIB.FREDtp&i(rename=(series_id=Series series_title=Title series_observation_start=Obs_Start 
                                           series_observation_end=Obs_End series_frequency=Frequency
                                           series_frequency_short=Frequency_short series_units_short=units
                                           series_seasonal_adjustment_short=seasonal_adjustment
                                           series_popularity=popularity) 
              keep=  series_id series_title series_observation_start series_observation_end series_frequency series_frequency_short series_units_short series_seasonal_adjustment_short series_popularity);
              retain series_id series_titl eseries_observation_start series_observation_end series_frequency series_frequency_short series_units_short series_seasonal_adjustment_short series_popularity;
                set outtp&i..series;
             run;
  
             %if &debug=ON %then %do;
                 proc contents data=FREDLIB.FREDtp&i; run;
                 proc print data=FREDLIB.FREDtp&i; run;
             %end;
          %end;
       %end;

       /*merge header information */
       %if &nseries_org>1 %then %do;  /* multiple time series */
          /* for merging the first two datasets */
          %do i=1 %to &nseries_org;
             %if &i~=1 and &i=2 %then %do;
                data FREDLIB.merged_freq;
                  retain Series;
                  length Series $ 100;
                  length Title $ 256;
                  length Frequency $ 50;
                  length Frequency_short $10;
                  length Units $10;
                  length Seasonal_adjustment $4;
                  
                  %let temp = 1;
                  merge FREDLIB.FREDtp&temp FREDLIB.FREDtp&i ;
                  by Series;
                run;
             %end;
             /* for all subsequent merges */
             %else %if &i~=1 %then %do;
                data FREDLIB.merged_freq;
                  retain Series;
                  length Series $ 100;
                  length Title $ 256;
                  length Frequency $ 50;
                  length Frequency_short $10;
                  length Units $10;
                  length Seasonal_adjustment $4;
                
                  merge FREDLIB.merged_freq FREDLIB.FREDtp&i;
                  by Series;
                run;
             %end;
          %end; /* End DO loop */
          
          %if &debug=ON %then %do;
              proc contents data=FREDLIB.merged_freq; run;
              proc print data=FREDLIB.merged_freq; run;
          %end;

          data FREDLIB.&nrbmname;
             set FREDLIB.merged_freq; 
          run;
          
          %if &debug=ON %then %do;
              proc contents data=FREDLIB.&nrbmname; run;
              proc print data=FREDLIB.&nrbmname; run;
          %end;

          %let nobs = &nseries_org;
          %let nvars = 9;
       %end; 

       %goto screen_return;
    %end; /* end of seroiesid_keep is empty */

    /* there is at least one time series in keep list */
    %do j=1 %to &initNumIds;  /* used to be totalNumIds */

        %if &&markNum&j = 0 %then %goto fini_out; /* only read the available selected datasets */        

        data _null_ ;
           %global seriname&j;
           call symput("seriname&j",scan("&nrbids",&&markNum&j,""));
        run;
        
        /* CASE 1*/
        %if &output=1 %then %do;
           %let temp = &&seriname&j;

           %if &vinlent<=0 %then %do;
           
            data FREDLIB.&&nrbmname.&j( rename=( observation_realtime_start=realtime_start observation_realtime_end=realtime_end observation_value=&temp observation_date=date )
                      keep= observation_realtime_start observation_realtime_end observation_date observation_value);
               retain observation_realtime_start observation_realtime_end observation_date observation_value;

           %end;
           %else %do; /* vintage does not prepend the observation_ */
           
            data FREDLIB.&&nrbmname.&j( rename=( value=&temp )
                      keep= realtime_start realtime_end date value);
               retain realtime_start realtime_end date value;

           %end;

               set outxml&j..observation;
           run;

           %if &debug=ON %then %do;        
               proc contents data=FREDLIB.&&nrbmname.&j; run;
               proc print data=FREDLIB.&&nrbmname.&j; run;
           %end;
        %end; /* END OF CASE 1 */

        /* CASE 2a */  
   %case_2a:
        %if &vinlent>0 %then %goto case_2b;
        /* no vintage_dates, output=2 */
        %if (&output=2) %then %do;   /* when output=2 do the following */
           %if &debug=ON %then %put CASE 2a;

           %if &units='lin' %then %do;            
              %let todaydate=%sysfunc(today(),yymmddn8.);
              %let temp = &&seriname&j;
              %put temp is &temp;
              data FREDLIB.&&nrbmname.&j( rename=( observation_&temp._&todaydate=&temp observation_date=date )
                         keep= observation_date observation_&temp._&todaydate);
                 retain observation_date observation_&temp._&todaydate;
                 set outxml&j..observation;
              run;
             
              %if &debug=ON %then %do;
                  proc contents data=FREDLIB.&&nrbmname.&j; run;
                  proc print data=FREDLIB.&&nrbmname.&j; run;
              %end;
           %end;
           %else %do; /* for cases where &units!='lin' */
              %let todaydate=%sysfunc(today(),yymmddn8.);
              %let temp = &&seriname&j;
              %put temp2 is &temp;
              data FREDLIB.&&nrbmname.&j( rename=( observation_&&temp._&unitname._&todaydate=&temp observation_date=date )
                         keep= observation_date observation_&temp._&unitname._&todaydate);
                 retain observation_date observation_&temp._&unitname._&todaydate;
                 set outxml&j..observation;
              run;

              %if &debug=ON %then %do;
                  proc contents data=FREDLIB.&&nrbmname.&j; run;
                  proc print data=FREDLIB.&&nrbmname.&j; run;
              %end;
           %end;
        %end; /* END OF CASE 2a */
  
        /* CASE 2b */
  
   %case_2b:
        %if &vinlent<=0 %then %goto case_3;
           /* vintage_dates, output=2, and prefixattributes=NO means no renaming necessary */
        %if (&output=2) %then %do;   /* when output=2 do the following */
           %if &debug=ON %then %do;
              %put CASE 2b;
              %put totalVintages = &totalVintages;
           %end;

           %if &units='lin' %then %do;             
              data FREDLIB.&&nrbmname.&j( drop=observation_ORDINAL observations_ORDINAL );
                 set outxml&j..observation;
              run;

              %if &debug=ON %then %do;
                  proc contents data=FREDLIB.&&nrbmname.&j; run;
                  proc print data=FREDLIB.&&nrbmname.&j; run;
              %end;
           %end;
           %else %do; /* for cases where &units!='lin' */
              /* %let vin     = %sysfunc(compress(&vintage,-));        */
              /* %let vinlen  = %length(&vin);                         */
              /* %let vindate = %substr( &vin, 2 , %eval(&vinlen-2) ); */
              /* %let temp = &&seriname&j;                             */
              /* %put temp2 is &temp;                                  */
              data FREDLIB.&&nrbmname.&j( drop=observation_ORDINAL observations_ORDINAL );
                 /* retain date &temp._&unitname._&vindate;            */
                 set outxml&j..observation;
              run;
              
              %if &debug=ON %then %do;
                  proc contents data=FREDLIB.&&nrbmname.&j; run;
                  proc print data=FREDLIB.&&nrbmname.&j; run;
              %end;
           %end;
        %end; /* END OF CASE 2b */

        /* CASE 3 */

   %case_3:
        
        %if (&output=3) %then %do;   /* when output=3 or 4 do the following */
             %if &debug=ON %then %do;
                %put CASE 3;
                %put totalVintages = &totalVintages;
             %end;

             data FREDLIB.&&nrbmname.&j( drop=observation_ORDINAL observations_ORDINAL );
                set outxml&j..observation;
             run;

             %if &debug=ON %then %do;
                 proc contents data=FREDLIB.&&nrbmname.&j; run;
                 proc print data=FREDLIB.&&nrbmname.&j; run;
             %end;

        %end;
        
        /* CASE 4 */

   %case_4:

        %if (&output=4) %then %do; 
           %if &debug=ON %then %put CASE 4;        
           %let temp = &&seriname&j;
           %if &vinlent<=0 %then %do;
           
            data FREDLIB.&&nrbmname.&j( rename=( observation_value=&temp observation_date=date )
                      keep= observation_date observation_value);
               retain observation_date observation_value;

           %end;
           %else %do; /* vintage does not prepend the observation_ */
            %let totalVintages = 1; /* output 4 is initial release only */

            data FREDLIB.&&nrbmname.&j( rename=( value=&temp )
                      keep= date value);
               retain date value;

           %end;

               set outxml&j..observation;
           run;

           %if &debug=ON %then %do;        
               proc contents data=FREDLIB.&&nrbmname.&j; run;
               proc print data=FREDLIB.&&nrbmname.&j; run;
           %end;

        %end; /* END OF CASE 4 */
   %fini_out:
    %end;  /* end of %DO */


    %if &totalNumIds=1 %then %do;  /* single time series */

       %do i=1 %to &initNumIds;

          %if &&markNum&i = 0 %then %goto skip_data;
        
          data FREDLIB.&nrbmname;
             retain date;
             set FREDLIB.&&nrbmname.&i ;  /* since there is only one, its i */
          run;

          proc sort data=FREDLIB.&nrbmname;
             by date;
          run;

          %if &debug=ON %then %do;
              proc contents data=FREDLIB.&nrbmname; run;
              proc print data=FREDLIB.&nrbmname; run;
          %end;

          %if &vinlent>0 %then %do;  /* update nvars, since vintages has multiple series */
             %if (&output=1) %then %let nvars = %eval( &totalNumIds + 3 );
             %else %let nvars = %eval( &totalVintages + 1);
          %end;
          %else %do;
             %if (&output=1) %then %let nvars = %eval( &totalNumIds + 3 );
          %end;
          %goto one_n_done;  /* only go thru once */

%skip_data:
       %end;

%one_n_done:
    %end;  /* end totalNumIds is one */
    
    %if &totalNumids>1 %then %do;  /* multiple time series, used to be totalNumIds */
       %do i=1 %to &initNumIds;   /* used to be totalNumIds */
          %if &&markNum&i = 0 %then %goto skip_merge;

          proc sort data=FREDLIB.&&nrbmname.&i;
             by date;
          run;

          /*--- save i for first database selected ---------*/
          %if &&markNum&i = 1 %then %do;
              %let temp = &i;
          %end;

          /* for merging the first two datasets */
          %else %if &&markNum&i = 2 %then %do;
             data FREDLIB.merged_data;
             retain date;             
             merge FREDLIB.&&nrbmname.&temp FREDLIB.&&nrbmname.&i ;
             by date;
             run;
          %end;

          /* for all subsequent merges */
          %else %if &&markNum&i~=1 %then %do;
             data FREDLIB.merged_data;
             retain date;
             merge FREDLIB.merged_data FREDLIB.&&nrbmname.&i;
             by date;
             run;
          %end; 
%skip_merge:
       %end; /* End DO loop */
       
       %if &debug=ON %then %do;
           proc contents data=FREDLIB.merged_data; run;
           proc print data=FREDLIB.merged_data; run;
       %end;


       %obsnvars(FREDLIB.merged_data)

       data FREDLIB.&nrbmname;
          set FREDLIB.merged_data; 
       run;

       /* sort observations in DESCENDING order if sort='DESC' */
       %if &obssort = %str(desc) %then %do;
          proc sort data=FREDLIB.&nrbmname;
             by descending date;
          run;
       %end;

    %end; /* End IF initNumIds > 1*/


   %if &nseries_org=1 %then %do;  /* single time serie */  
      data FREDLIB.FREDtp1(rename=(series_id=Series series_title=Title series_observation_start=Obs_Start 
                           series_observation_end=Obs_End series_frequency=Frequency 
                           series_frequency_short=Frequency_short series_units_short=units
                           series_seasonal_adjustment_short=seasonal_adjustment
                           series_popularity=popularity)
         keep=  series_id series_title series_observation_start series_observation_end series_frequency series_frequency_short series_units_short series_seasonal_adjustment_short series_popularity);
         retain series_id series_title series_observation_start series_observation_end series_frequency series_frequency_short series_units_short series_seasonal_adjustment_short series_popularity;
         set outtp1.series;
      run;

      %if &debug=ON %then %do;
          proc contents data=FREDLIB.FREDtp1; run;
          proc print data=FREDLIB.FREDtp1; run;
      %end;

      data FREDLIB.OUTCONT_&nrbmname;
         set FREDLIB.FREDtp1; 
      run;

      proc contents data=FREDLIB.OUTCONT_&nrbmname; run;
      proc print data=FREDLIB.OUTCONT_&nrbmname; run;

   %end;

   %if &nseries_org>1 %then %do;  /* multiple time series */
      %do i=1 %to &nseries_org; 
         data FREDLIB.FREDtp&i(rename=(series_id=Series series_title=Title series_observation_start=Obs_Start 
                                       series_observation_end=Obs_End series_frequency=Frequency
                                       series_frequency_short=Frequency_short series_units_short=units
                                       series_seasonal_adjustment_short=seasonal_adjustment
                                       series_popularity=popularity) 
          keep=  series_id series_title series_observation_start series_observation_end series_frequency series_frequency_short series_units_short series_seasonal_adjustment_short series_popularity);
          retain series_id series_title series_observation_start series_observation_end series_frequency series_frequency_short series_units_short series_seasonal_adjustment_short series_popularity;
            set outtp&i..series;
         run; 

         %if &debug=ON %then %do;
             proc contents data=FREDLIB.FREDtp&i; run;
             proc print data=FREDLIB.FREDtp&i; run;
         %end;
      %end;
   %end;

   /*merge header information */
   %if &nseries_org>1 %then %do;  /* multiple time series */
      /* for merging the first two datasets */
      %do i=1 %to &nseries_org;
         %if &i~=1 and &i=2 %then %do;
            data FREDLIB.merged_freq;
              retain Series;
              length Series $ 100;
              length Title $ 256;
              length Frequency $ 50;
              length Frequency_short $10;
              length Units $10;
              length Seasonal_adjustment $4;
            
              %let temp = 1;
              merge FREDLIB.FREDtp&temp FREDLIB.FREDtp&i ;
              by Series;
            run;
         %end;
         /* for all subsequent merges */
         %else %if &i~=1 %then %do;
            data FREDLIB.merged_freq;
              retain Series;
              length Series $ 100;
              length Title $ 256;
              length Frequency $ 50;
              length Frequency_short $10;
              length Units $10;
              length Seasonal_adjustment $4;
           
              merge FREDLIB.merged_freq FREDLIB.FREDtp&i;
              by Series;
            run;
         %end;
      %end; /* End DO loop */

      %if &debug=ON %then %do;      
          proc contents data=FREDLIB.merged_freq; run;
          proc print data=FREDLIB.merged_freq; run;
      %end;

      data FREDLIB.OUTCONT_&nrbmname;
         set FREDLIB.merged_freq; 
      run;

      proc contents data=FREDLIB.OUTCONT_&nrbmname; run;
      proc print data=FREDLIB.OUTCONT_&nrbmname; run;

   %end;
   %goto screen_return;

%fred_unique_true:
    /* URL= option, we need to capture OUTTPU.MEMNAM1 into FREDLIB.MEMNAM1 */

    %if &nrbfredref EQ %then %do;
        %put ERROR: FREDREF= parameter is required for FRED_FILEPUTDS macro.;
        %goto clean_up;
    %end;

    %if &memname EQ %then %do;
        %put ERROR: MEMNAME= parameter is required for URL= option in FRED_FILEPUTDS macro.;
        %goto clean_up;
    %end;

    /*---- strip off the 's' for the &memname ----*/
    %let memlen1 = %eval( %length(&memname) -1 );
    %let memnam1 = %substr( &memname, 1, &memlen1 );

    %if &memnam1=categorie %then %let memnam1 = %str(category);
 
    proc datasets lib=OUTTPU; run;
   
    libname FREDLIB &nrbfredref;

    data FREDLIB.XFREDtpu;
       set OUTTPU.&memnam1;
    run;

    %obsnvars(FREDLIB.XFREDtpu)

    data FREDLIB.&memnam1;
       set OUTTPU.&memnam1;
    run;

    data FREDLIB.&memname;
       set OUTTPU.&memname;
    run;
     
%screen_return:
    
    %let fred_macro_nobs=&nobs;
    %let fred_macro_nvars=%eval(&nvars);
    %let fred_macro_status=0;

%clean_up:
   /*---- this will be clean-up later, but for now -----------*/
   /*---- see what is there is the work that was created -----*/
   /*
    proc datasets lib=work;
    run;                  
    proc datasets lib=FREDLIB;
    run;
   */
   *-- unconditionally cut notes on --;
   options nonotes;
   %if &debug=OFF %then %do;
      proc printto log=log;
      run;

    /* -- s1111072, unresolved macros...*/
    /* -- filename newlog "newlog.tmp"; */
    /* -- %ridfile(newlog.tmp);         */

      %let rc = %sysfunc(fdelete(newlog));
   %end;

   options notes; options validvarname=any;

%mend fred_fileputds;
